/** 
 * 
 * Copyright 2004 Hiram Chirino
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.codehaus.activemq.router.filter.mockrunner;



/**
 * An expression which performs an operation on two expression values.
 * 
 * Alwin Ibba: Changed package
 * 
 * @version $Revision: 1.1 $
 */
abstract public class BinaryExpression implements Expression {
    protected Expression left;
    protected Expression right;

    public BinaryExpression(Expression left, Expression right) {
        this.left = left;
        this.right = right;
    }

    public Expression getLeft() {
        return left;
    }

    public Expression getRight() {
        return right;
    }
    
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "("+left.toString()+" "+getExpressionSymbol()+" "+right.toString()+")";
    }

    /**
     * TODO: more efficient hashCode()
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return toString().hashCode();
	}
	
	/**
     * TODO: more efficient hashCode()
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object o) {
		
		if( o==null || !this.getClass().equals(o.getClass()) )
			return false;
		return toString().equals( o.toString() );
		
	}
    
    /**
     * Returns the symbol that represents this binary expression.  For example, addition is 
     * represented by "+"
     * 
     * @return
     */
    abstract public String getExpressionSymbol();

	/**
	 * @param expression
	 */
	public void setRight(Expression expression) {
		right = expression;
	}

	/**
	 * @param expression
	 */
	public void setLeft(Expression expression) {
		left = expression;
	}

}
